@@ -13,6 +13,7 @@ from collections import Iterable
 from contextlib import contextmanager
 
 import netifaces
+import requests
 import six
 from cryptography.fernet import Fernet
 from flask import current_app
@@ -393,3 +394,9 @@ def session_scope(session=None):
 
 def format_exception(exc: Exception):
     return ''.join(traceback.format_exception(exc, exc, exc.__traceback__))
+
+def str_resp(resp: requests.Response):
+    try:
+        return resp.json()
+    except ValueError:
+        return resp.text
\ No newline at end of file
