@@ -1,5 +1,6 @@
 import base64
 import os
+import zlib
 
 from flask import request, current_app
 from flask_jwt_extended import jwt_required
@@ -17,8 +18,8 @@ from dimensigon.web.json_schemas import log_post, logs_post, log_patch
 
 class LogList(Resource):
 
-    @securizer
     @jwt_required
+    @securizer
     @forward_or_dispatch()
     def get(self):
         query = filter_query(Log, request.args)
@@ -31,8 +32,8 @@ class LogList(Resource):
     @lock_catalog
     def post(self):
         data = request.get_json()
-        source_server = Server.query.get_or_404(data.pop('src_server_id'))
-        destination_server = Server.query.get_or_404(data.pop('dst_server_id'))
+        source_server = Server.query.get_or_raise(data.pop('src_server_id'))
+        destination_server = Server.query.get_or_raise(data.pop('dst_server_id'))
         if source_server == destination_server:
             return {'error': 'source and destination must be different'}, 400
         if 'mode' in data:
@@ -48,41 +49,49 @@ class LogList(Resource):
 
 class LogResource(Resource):
 
-    @securizer
     @jwt_required
+    @securizer
     @forward_or_dispatch()
     def get(self, log_id):
-        return Log.query.get_or_404(log_id).to_json()
+        return Log.query.get_or_raise(log_id).to_json()
 
-    @securizer
     @jwt_required
+    @securizer
     @forward_or_dispatch()
     @validate_schema(log_post)
     def post(self, log_id):
-        log = Log.query.get_or_404(log_id)
+        log = Log.query.get_or_raise(log_id)
         data = request.get_json()
         file = data.get('file')
         if log.mode in (Mode.REPO_MIRROR, Mode.REPO_ROOT):
             file = file.format(
-                LOG_REPO=os.path.join(current_app.dm.config.config_dir, defaults.LOG_REPO,
+                LOG_REPO=os.path.join(current_app.dm.config.config_dir, defaults.LOG_SENDER_REPO,
                                       clean_string(log.source_server.name)))
-        data_log = base64.b64decode(data.get('data').encode('ascii'))
-        try:
-            if not os.path.exists(os.path.dirname(file)):
+        if data.get('compress', False):
+            data_log = zlib.decompress(base64.b64decode(data.get('data').encode('ascii')))
+        else:
+            data_log = base64.b64decode(data.get('data').encode('ascii'))
+
+        if not os.path.exists(os.path.dirname(file)):
+            try:
                 os.makedirs(os.path.dirname(file))
+            except PermissionError:
+                raise errors.GenericError(f"Permission denied creating '{os.makedirs(os.path.dirname(file))}'", 500)
+        try:
             with open(file, 'ab') as fh:
                 fh.write(data_log)
         except Exception as e:
-            return {"error": str(e)}
+            raise errors.GenericError(f"{e}", 500)
+
         return {'offset': os.path.getsize(file)}
 
-    @securizer
     @jwt_required
+    @securizer
     @forward_or_dispatch()
     @validate_schema(log_patch)
     @lock_catalog
     def patch(self, log_id):
-        log = Log.query.get_or_404(log_id)
+        log = Log.query.get_or_raise(log_id)
         data = request.get_json()
         if 'include' in data and log.include != data.get('include'):
             log.include = data.get('include')
@@ -102,12 +111,12 @@ class LogResource(Resource):
             return {}, 204
         return {}, 202
 
-    @securizer
     @jwt_required
+    @securizer
     @forward_or_dispatch()
     @lock_catalog
     def delete(self, log_id):
-        log = Log.query.get_or_404(log_id)
+        log = Log.query.get_or_raise(log_id)
         log.delete()
         db.session.commit()
         return {}, 204
